"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const macroable_1 = require("macroable");
const utils_1 = require("@poppinss/utils");
const BriskRoute_1 = require("./BriskRoute");
const Resource_1 = require("./Resource");
function missingRouteName() {
    return new utils_1.Exception('All routes inside a group must have names before calling Route.group.as', 500, 'E_MISSING_ROUTE_NAME');
}
class RouteGroup extends macroable_1.Macroable {
    constructor(routes) {
        super();
        this.routes = routes;
    }
    _invoke(route, method, params) {
        if (route instanceof Resource_1.RouteResource) {
            route.routes.forEach((child) => this._invoke(child, method, params));
            return;
        }
        if (route instanceof RouteGroup) {
            route[method](...params);
            return;
        }
        if (route instanceof BriskRoute_1.BriskRoute) {
            if (route.route) {
                if (method === 'as' && !route.route.name) {
                    throw missingRouteName();
                }
                route.route[method](...params);
            }
            return;
        }
        if (method === 'as' && !route.name) {
            throw missingRouteName();
        }
        route[method](...params);
    }
    where(param, matcher) {
        this.routes.forEach((route) => this._invoke(route, 'where', [param, matcher]));
        return this;
    }
    prefix(prefix) {
        this.routes.forEach((route) => this._invoke(route, 'prefix', [prefix]));
        return this;
    }
    domain(domain) {
        this.routes.forEach((route) => this._invoke(route, 'domain', [domain]));
        return this;
    }
    as(name) {
        this.routes.forEach((route) => this._invoke(route, 'as', [name, true]));
        return this;
    }
    middleware(middleware) {
        this.routes.forEach((route) => this._invoke(route, 'middleware', [middleware, true]));
        return this;
    }
    namespace(namespace) {
        this.routes.forEach((route) => this._invoke(route, 'namespace', [namespace]));
        return this;
    }
}
exports.RouteGroup = RouteGroup;
RouteGroup._macros = {};
RouteGroup._getters = {};
